package com.example.greenwatch;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.AsyncTask;
import android.os.Bundle;
import android.widget.TextView;

import java.text.SimpleDateFormat;
import java.util.Date;

public class Mikrofon extends AppCompatActivity {

    private AufnahmeTask aufnahmeTask;
    private TextView textViewMaxAmp;
    private TextView textViewMaxAmpdb;
    private TextView textViewZeitstempel;
    private static final int RECHTEANFORDERUNG_MIKROFON = 1;

    private boolean detection_mic = false;
    private String zeit = "0";
    private boolean kalibrierung_do = true;
    private float threshold = 40;
    private float sensitivity = 5;
    private final static String KEY_WERT = "KEY_WERT_" + Mikrofon.class.getSimpleName();
    private final static String KEY_KALIBRIERUNG = "KEY_KALIBRIERUNG_" + Mikrofon.class.getSimpleName();




    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_mikrofon);
        textViewMaxAmp = (TextView) findViewById(R.id.textViewMaxAmp);
        textViewMaxAmpdb = (TextView) findViewById(R.id.textViewMaxAmpdb);
        textViewZeitstempel = (TextView) findViewById(R.id.textViewZeitstempel);
        //kalibrierung_do = true;


    }


    @Override
    protected void onResume() {
        super.onResume();
        SharedPreferences p = getPreferences(Context.MODE_PRIVATE);
        threshold = p.getFloat(KEY_WERT, 40);
        kalibrierung_do = p.getBoolean(KEY_KALIBRIERUNG, true);

        if (!istZugriffAufMikrofonErlaubt()) {
            zugriffAufMikrofonAnfordern();
        }
        if (istZugriffAufMikrofonErlaubt()) {
            aufnahmeTask = new AufnahmeTask();
            aufnahmeTask.execute();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        SharedPreferences p = getPreferences(Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = p.edit();
        editor.putFloat(KEY_WERT, threshold);
        if(isFinishing()==true){
            editor.putBoolean(KEY_KALIBRIERUNG, true);
        }else {
            editor.putBoolean(KEY_KALIBRIERUNG, false);
        }
        editor.commit();
        if(aufnahmeTask!=null) {
            aufnahmeTask.cancel(true);
            aufnahmeTask = null;
        }
    }

    private boolean istZugriffAufMikrofonErlaubt() {
        if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
            return false;
        } else {
            return true;
        }
    }

    private void zugriffAufMikrofonAnfordern() {
        ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.RECORD_AUDIO}, RECHTEANFORDERUNG_MIKROFON);
    }

    public static String Zeitpunkt(){
        Long tslong = System.currentTimeMillis();
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        Date date = new Date(tslong);
        String resultdate = sdf.format(date);

        return resultdate;
    }


    class AufnahmeTask extends AsyncTask<Long, Verarbeitungsergebnis, Void> {
        private AudioRecord recorder;
        private final int sampleRateInHz = 44100;
        private final int channelConfig = AudioFormat.CHANNEL_IN_MONO;
        private final int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
        private int minPufferGroesseInBytes;
        private int pufferGroesseInBytes;
        private RingPuffer ringPuffer = new RingPuffer(10);
        private boolean erlaubt_rise = true;
        private boolean erlaubt_fall = true;

        AufnahmeTask() {
            minPufferGroesseInBytes = AudioRecord.getMinBufferSize(sampleRateInHz, channelConfig, audioFormat);
            pufferGroesseInBytes = minPufferGroesseInBytes * 2;
            recorder = new AudioRecord(MediaRecorder.AudioSource.MIC, sampleRateInHz, channelConfig, audioFormat, pufferGroesseInBytes);
        }
        //hi
        public float kalibrieren(short[] puffer){
            try {
                Thread.sleep(3000);

            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            int anzahlIterationen = 100;
            float sum = 0;
            for(int i = 0; i < anzahlIterationen; i++){
                int z = recorder.read(puffer, 0, puffer.length);
                Verarbeitungsergebnis kalibrierung = verarbeiten(puffer, z);
                sum += (float) kalibrierung.db;
                System.out.println("Aktueller_Wert Kalibrierung" +kalibrierung.db);
            }
            System.out.println("Summe:"+sum);
            return sum/anzahlIterationen;
        }

        public void Detektion(Verarbeitungsergebnis ergebnis){
            if(ergebnis.db >= (threshold+sensitivity)){
                detection_mic = true;
                erlaubt_fall = true;
                while(erlaubt_rise){
                    erlaubt_rise = false;
                    zeit = Zeitpunkt() + " - " + ergebnis.db + " - " + (threshold+sensitivity); //Überschreiben des Zeitpunkts beim Erkennen der Detektion
                    //nur zum Testen in zeit noch ergebnis.db und threshold ausgegeben -> muss am Ende nur Zeitpunkt() rein, Rest weg
                }
            } else if (ergebnis.db <= (threshold)) {
                detection_mic = false;
                erlaubt_rise = true;
                while(erlaubt_fall){
                    erlaubt_fall = false;
                    zeit = Zeitpunkt(); //erneutes Überschreiben des Zeitpunkts beim Verschwinden der Detektion
                }
            }
        }
        @Override
        protected Void doInBackground(Long... params) {

            recorder.startRecording();
            short[] puffer = new short[pufferGroesseInBytes / 2];
            GleitenderMittelwert gleitenderMittelwertdB = new GleitenderMittelwert(0.2f);
            float db = 0;


            //kalibrierung
            if(kalibrierung_do){
                threshold = kalibrieren(puffer);
                kalibrierung_do = false;
            }


            for (; ; ) {
                if (isCancelled()) {
                    break;
                } else {
                    //ergebnis ermitteln
                    int n = recorder.read(puffer, 0, puffer.length);
                    Verarbeitungsergebnis ergebnis = verarbeiten(puffer, n);

                    //Mittelwertberechnnung
                    db = (float) ergebnis.db;
                    db = gleitenderMittelwertdB.mittel(db);
                    ergebnis.db = (int) db;

                    //Db Wert mit Schwellwert vergleichen und Warnung setzen
                    Detektion(ergebnis);
                    publishProgress(ergebnis);

                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
            recorder.release();
            return null;
        }

        private Verarbeitungsergebnis verarbeiten(short[] daten, int n) {
            String status;
            short maxAmp = -1;
            int db = 0, db_wert = 0;

            if (n == AudioRecord.ERROR_INVALID_OPERATION) {
                status = "ERROR_INVALID_OPERATION";
            } else if (n == AudioRecord.ERROR_BAD_VALUE) {
                status = "ERROR_BAD_VALUE";
            } else {
                status = "OK";
                short max = 0;
                for (int i = 0; i < n; i++) {
                    if (daten[i] > max) {
                        max = daten[i];
                    }
                }
                ringPuffer.hinzufuegen(max);
                maxAmp = ringPuffer.maximum();
            }
            //Umwandlung Amplitudenwert in dB
            db_wert = (int) (20*Math.log10(maxAmp));
            if (db_wert > 0){
                db = db_wert;
            }


            return new Verarbeitungsergebnis(status, maxAmp, db);
        }

        /////////////////////////////////////////////
        @Override
        protected void onProgressUpdate(Verarbeitungsergebnis... progress) {
            super.onProgressUpdate(progress);


            System.out.println(detection_mic);
            System.out.println(zeit);
            textViewZeitstempel.setText(zeit);

            textViewMaxAmpdb.setText("" + progress[0].db);
            textViewMaxAmp.setText("" + progress[0].maxAmp);

        }
    }
    /////////////////////////////////////////////

}
